@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtBluetooth module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qbluetoothservicediscoveryagent.h"
 #include "qbluetoothservicediscoveryagent_p.h"
@@ -79,6 +43,13 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_WINDOWS)
 #define TYPE_STRING 37
 #define TYPE_SEQUENCE 53
 
+// Helper to reverse given uchar array
+static void reverseArray(uchar data[], size_t length)
+{
+    for (size_t i = length; i > length/2; i--)
+        std::swap(data[length - i], data[i - 1]);
+}
+
 class QWinRTBluetoothServiceDiscoveryWorker : public QObject
 {
     Q_OBJECT
@@ -91,7 +62,6 @@ public:
 Q_SIGNALS:
     void serviceFound(quint64 deviceAddress, const QBluetoothServiceInfo &info);
     void scanFinished(quint64 deviceAddress);
-    void scanCanceled();
     void errorOccured();
 
 private:
@@ -289,6 +259,8 @@ void QWinRTBluetoothServiceDiscoveryWorker::processServiceSearchResult(quint64 a
                 GUID value;
                 hr = dataReader->ReadGuid(&value);
                 Q_ASSERT_SUCCEEDED(hr);
+                // The latter 8 bytes are in reverse order
+                reverseArray(value.Data4, sizeof(value.Data4)/sizeof(value.Data4[0]));
                 const QBluetoothUuid uuid(value);
                 info.setAttribute(key, uuid);
                 qCDebug(QT_BT_WINDOWS) << "UUID" << uuid << "KEY" << Qt::hex << key << "TYPE" << Qt::dec << type << "UUID" << Qt::hex << uuid;
@@ -402,7 +374,8 @@ QBluetoothServiceInfo::Sequence QWinRTBluetoothServiceDiscoveryWorker::readSeque
             GUID b;
             hr = dataReader->ReadGuid(&b);
             Q_ASSERT_SUCCEEDED(hr);
-
+            // The latter 8 bytes are in reverse order
+            reverseArray(b.Data4, sizeof(b.Data4)/sizeof(b.Data4[0]));
             const QBluetoothUuid uuid(b);
             result.append(QVariant::fromValue(uuid));
             remainingLength -= sizeof(GUID);
@@ -468,13 +441,15 @@ QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(
       singleDevice(false),
       q_ptr(qp)
 {
+    mainThreadCoInit(this);
     // TODO: use local adapter for discovery. Possible?
     Q_UNUSED(deviceAdapter);
 }
 
 QBluetoothServiceDiscoveryAgentPrivate::~QBluetoothServiceDiscoveryAgentPrivate()
 {
-    stop();
+    releaseWorker();
+    mainThreadCoUninit(this);
 }
 
 void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &address)
@@ -488,8 +463,6 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
         this, &QBluetoothServiceDiscoveryAgentPrivate::processFoundService, Qt::QueuedConnection);
     connect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanFinished,
         this, &QBluetoothServiceDiscoveryAgentPrivate::onScanFinished, Qt::QueuedConnection);
-    connect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanCanceled,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanCanceled, Qt::QueuedConnection);
     connect(worker, &QWinRTBluetoothServiceDiscoveryWorker::errorOccured,
         this, &QBluetoothServiceDiscoveryAgentPrivate::onError, Qt::QueuedConnection);
     worker->start();
@@ -497,20 +470,9 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
 
 void QBluetoothServiceDiscoveryAgentPrivate::stop()
 {
-    if (!worker)
-        return;
-
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::serviceFound,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::processFoundService);
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanFinished,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanFinished);
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanCanceled,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanCanceled);
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::errorOccured,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onError);
-    // mWorker will delete itself as soon as it is done with its discovery
-    worker = nullptr;
-    setDiscoveryState(Inactive);
+    releaseWorker();
+    Q_Q(QBluetoothServiceDiscoveryAgent);
+    emit q->canceled();
 }
 
 void QBluetoothServiceDiscoveryAgentPrivate::processFoundService(quint64 deviceAddress, const QBluetoothServiceInfo &info)
@@ -559,26 +521,12 @@ void QBluetoothServiceDiscoveryAgentPrivate::processFoundService(quint64 deviceA
 
 void QBluetoothServiceDiscoveryAgentPrivate::onScanFinished(quint64 deviceAddress)
 {
-    Q_Q(QBluetoothServiceDiscoveryAgent);
-    bool deviceFound;
-    for (const QBluetoothDeviceInfo &deviceInfo : qAsConst(discoveredDevices)) {
-        if (deviceInfo.address().toUInt64() == deviceAddress) {
-            deviceFound = true;
-            discoveredDevices.removeOne(deviceInfo);
-            if (discoveredDevices.isEmpty())
-                setDiscoveryState(Inactive);
-            break;
-        }
-    }
-    Q_ASSERT(deviceFound);
-    stop();
-    emit q->finished();
-}
-
-void QBluetoothServiceDiscoveryAgentPrivate::onScanCanceled()
-{
-    Q_Q(QBluetoothServiceDiscoveryAgent);
-    emit q->canceled();
+    // The scan for a device's services has finished. Disconnect the
+    // worker and call the baseclass function which starts the scan for
+    // the next device if there are any unscanned devices left (or finishes
+    // the scan if none left)
+    releaseWorker();
+    _q_serviceDiscoveryFinished();
 }
 
 void QBluetoothServiceDiscoveryAgentPrivate::onError()
@@ -590,6 +538,21 @@ void QBluetoothServiceDiscoveryAgentPrivate::onError()
     emit q->errorOccurred(error);
 }
 
+void QBluetoothServiceDiscoveryAgentPrivate::releaseWorker()
+{
+    if (!worker)
+        return;
+
+    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::serviceFound,
+        this, &QBluetoothServiceDiscoveryAgentPrivate::processFoundService);
+    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanFinished,
+        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanFinished);
+    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::errorOccured,
+        this, &QBluetoothServiceDiscoveryAgentPrivate::onError);
+    // mWorker will delete itself as soon as it is done with its discovery
+    worker = nullptr;
+}
+
 QT_END_NAMESPACE
 
 #include <qbluetoothservicediscoveryagent_winrt.moc>
